@@ -17,7 +17,7 @@
 #include "libfs.h"
 #include "userconfig.h"
 
-#define DELIMITER ':'
+#define DELIMITER ';'
 
 typedef struct type_wrap {
 	char *cgiroot;
