@@ -570,6 +570,11 @@ int parse_request(t_session *session, int total_bytes) {
 	if ((session->request_uri = strdup(session->uri)) == NULL) {
 		return -1;
 	}
+	if (session->host->secure_url) {
+		if (strstr(session->request_uri, "%00") != NULL) {
+			return 403;
+		}
+	}
 
 	/* Protocol version
 	 */
@@ -837,7 +842,7 @@ int serve_client(t_session *session) {
 	 */
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
 		for (i = 0; i < session->host->rewrite_rules.size; i++) {
-			if ((result = rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri)) == RW_ERROR) {
+			if ((result = rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri, session->host->website_root)) == RW_ERROR) {
 				return 500;
 			}
 			if (new_uri != NULL) {
@@ -851,6 +856,11 @@ int serve_client(t_session *session) {
 				session->uri = new_uri;
 			}
 
+			if (result == RW_REDIRECT) {
+				session->location = strdup(new_uri);
+				session->cause_of_301 = location;
+				return 301;
+			}
 			if (result == RW_DENY_ACCESS) {
 				log_error(session, "access denied via URL rewrite rule");
 				return 403;
@@ -874,7 +884,9 @@ int serve_client(t_session *session) {
 	url_decode(session->uri);
 	session->uri_len = strlen(session->uri);
 	if ((session->vars != NULL) && (session->host->secure_url)) {
-		remove_dangerous_chars(session->vars);
+		if (forbidden_chars_present(session->vars)) {
+			return 403;
+		}
 	}
 
 	if (valid_uri(session->uri) == false) {
@@ -1216,6 +1228,7 @@ void connection_handler(t_session *session) {
 	} else {
 		remove_client(session, true);
 	}
+
 	/* Client session ends here */
 }
 
@@ -1696,26 +1709,28 @@ int run_server(t_settings *settings) {
 
 	/* Redirecting I/O to /dev/null
 	 */
-	if (close(STDIN_FILENO) == -1) {
-		fprintf(stderr, "Warning: error closing STDIN\n");
-	} else if (open("/dev/null", O_RDONLY) == -1) {
-		fprintf(stderr, "Error redirecting stdin\n");
-		return EXIT_FAILURE;
-	}
+	if (settings->daemon) {
+		if (close(STDIN_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDIN\n");
+		} else if (open("/dev/null", O_RDONLY) == -1) {
+			fprintf(stderr, "Error redirecting stdin\n");
+			return EXIT_FAILURE;
+		}
 #ifndef DEBUG
-	if (close(STDOUT_FILENO) == -1) {
-		fprintf(stderr, "Warning: error closing STDOUT\n");
-	} else if (open("/dev/null", O_WRONLY) == -1) {
-		fprintf(stderr, "Error redirecting stdout\n");
-		return EXIT_FAILURE;
-	}
-	if (close(STDERR_FILENO) == -1) {
-		fprintf(stderr, "Warning: error closing STDERR\n");
-	} else if (open("/dev/null", O_WRONLY) == -1) {
-		log_string(config->system_logfile, "Error redirecting stderr\n");
-		return EXIT_FAILURE;
-	}
+		if (close(STDOUT_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDOUT\n");
+		} else if (open("/dev/null", O_WRONLY) == -1) {
+			fprintf(stderr, "Error redirecting stdout\n");
+			return EXIT_FAILURE;
+		}
+		if (close(STDERR_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDERR\n");
+		} else if (open("/dev/null", O_WRONLY) == -1) {
+			log_string(config->system_logfile, "Error redirecting stderr\n");
+			return EXIT_FAILURE;
+		}
 #endif
+	}
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" started");
 
