@@ -168,7 +168,7 @@ int parse_ip_port(char *line, t_ip_addr *ip, int *port) {
 		s_port = remove_spaces(s_port + 1);
 	} else
 #endif
-	if (split_string(line, &s_ip, &s_port, ' ') != 0) {
+	{
 		s_port = line + strlen(line);
 		do {
 			if (s_port <= line) {
