@@ -14,6 +14,91 @@ void sfree(void *ptr) {
 	}
 }
 
+/*---< headerlist >-----------------------------------------------------------*/
+
+/* Parse the HTTP headerfields from the received request.
+ */
+t_headerfield *parse_headerfields(char *line) {
+	t_headerfield *first, *headerfield;
+	char *value;
+
+	if (*line == '\0') {
+		return NULL;
+	}
+
+	if ((first = headerfield = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
+		return NULL;
+	}
+	headerfield->data = line;
+	headerfield->next = NULL;
+	while (*line != '\0') {
+		if (*line == '\r') {
+			*line = '\0';
+			if (*(line + 1) == '\n') {
+				if ((*(line + 2) != '\r') && (*(line + 2) != '\0')) {
+					if ((headerfield->next = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
+						return first;
+					}
+					headerfield = headerfield->next;
+					headerfield->next = NULL;
+					headerfield->data = line + 2;
+				} else {
+					break;
+				}
+			}
+		}
+		line++;
+	}
+
+	headerfield = first;
+	while (headerfield != NULL) {
+		if ((value = strchr(headerfield->data, ':')) != NULL) {
+			do {
+				value++;
+			} while ((*value == ' ') && (*value != '\0'));
+			headerfield->value_offset = (value - headerfield->data);
+		} else {
+			headerfield->value_offset = 0;
+		}
+		headerfield = headerfield->next;
+	}
+
+	return first;
+}
+
+/* Search for a headerfield and return its value.
+ */
+char *get_headerfield(char *key, t_headerfield *headerfields) {
+	char *retval = NULL;
+	int len;
+
+	len = strlen(key);
+	while (headerfields != NULL) {
+		if (strncasecmp(headerfields->data, key, len) == 0) {
+			retval = headerfields->data + headerfields->value_offset;
+			break;
+		}
+		headerfields = headerfields->next;
+	}
+
+	return retval;
+}
+
+/* free() a list of headerfields.
+ */
+t_headerfield *remove_headerfields(t_headerfield *headerfields) {
+	t_headerfield *remove;
+
+	while (headerfields != NULL) {
+		remove = headerfields;
+		headerfields = headerfields->next;
+
+		free(remove);
+	}
+
+	return NULL;
+}
+
 /*---< charlist >-------------------------------------------------------------*/
 
 void init_charlist(t_charlist *list) {
