@@ -8,6 +8,13 @@
 typedef enum { deny, allow, pwd, unspecified } t_access;
 typedef enum { tc_data, tc_charlist, tc_accesslist, tc_keyvalue } t_tempdata_type;
 
+typedef struct type_headerfield {
+	char *data;
+	int  value_offset;
+
+	struct type_headerfield *next;
+} t_headerfield;
+
 typedef struct type_charlist {
 	int  size;
 	char **item;
@@ -45,6 +52,10 @@ typedef struct type_tempdata {
 
 void sfree(void* ptr);
 
+t_headerfield *parse_headerfields(char *line);
+char *get_headerfield(char *key, t_headerfield *headerfields);
+t_headerfield *remove_headerfields(t_headerfield *headerfields);
+
 void init_charlist(t_charlist *list);
 int  parse_charlist(char *value, t_charlist *list);
 void copy_charlist(t_charlist *dest, t_charlist *src);
