@@ -245,14 +245,22 @@ int ssl_send(SSL *ssl, char *buffer, unsigned int length) {
 	return result;
 }
 
-int ssl_close(SSL *ssl) {
-	return SSL_shutdown(ssl);
+int ssl_close(SSL **ssl) {
+	int result;
+
+	result = SSL_shutdown(*ssl);
+	SSL_free(*ssl);
+	*ssl = NULL;
+
+	ERR_remove_state(0);
+
+	return result;
 }
 
-void ssl_cleanup(SSL_CTX *sslContext) {
+void ssl_cleanup(SSL_CTX *context) {
 	int i;
 
-	SSL_CTX_free(sslContext);
+	SSL_CTX_free(context);
   
 	for (i = 0; i < lockcount; i++) {
 		pthread_mutex_destroy(&locks[i]);
