@@ -198,31 +198,31 @@ void url_decode(char *line) {
 	}
 }
 
-/* Remove characters with ASCII value < 32.
+/* Scan for characters with ASCII value < 32.
  */
-void remove_dangerous_chars(char *line) {
+bool forbidden_chars_present(char *line) {
 	short low, high;
-	char *dest = line;
 
-	if (line != NULL) {
-		while (*line != '\0') {
-			if ((*line > 0) && (*line < 32)) {
-				line++;
-				continue;
-			} else if (*line == '%') {
-				if ((high = hex_to_int(*(line + 1))) != -1) {
-					if ((low = hex_to_int(*(line + 2))) != -1) {
-						if (((high<<4) + low) < 32) {
-							line += 3;
-							continue;
-						}
+	if (line == NULL) {
+		return false;
+	}
+
+	while (*line != '\0') {
+		if ((*line > 0) && (*line < 32)) {
+			return true;
+		} else if (*line == '%') {
+			if ((high = hex_to_int(*(line + 1))) != -1) {
+				if ((low = hex_to_int(*(line + 2))) != -1) {
+					if (((high<<4) + low) < 32) {
+						return true;
 					}
 				}
 			}
-			*(dest++) = *(line++);
 		}
-		*dest = *line;
+		line++;
 	}
+
+	return false;
 }
 
 /* Prevent Command injection.
