@@ -4,8 +4,10 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <grp.h>
+#include <time.h>
 #include "alternative.h"
 #include "libstr.h"
 #include "liblist.h"
@@ -13,6 +15,7 @@
 
 #define MAX_CONFIG_LINE KILOBYTE
 #define DELIMITER ';'
+#define TIMESTAMP_SIZE    40
 
 /* Type definitions
  */
@@ -37,10 +40,30 @@ typedef struct {
 
 /* Code
  */
-int run_php_fcgi(t_php_fcgi *php_fcgi) {
+
+void log_string(char *mesg) {
+	FILE *fp;
+	time_t t;
+	struct tm *s;
+	char str[TIMESTAMP_SIZE];
+
+	time(&t);
+	s = localtime(&t);
+	str[TIMESTAMP_SIZE - 1] = '\0';
+	strftime(str, TIMESTAMP_SIZE - 1, "%a %d %b %Y %T %z|", s);
+
+	if ((fp = fopen(LOG_DIR"/php-fcgi.log", "a")) != NULL) {
+		fprintf(fp, "%s%s\n", str, mesg);
+		fclose(fp);
+	}
+}
+
+int run_php_fcgi(t_php_fcgi *php_fcgi, FILE *pid_fp) {
 	pid_t pid;
 
 	if ((pid = fork()) == 0) {
+		fclose(pid_fp);
+
 		if (php_fcgi->chroot != NULL) {
 			if (chdir(php_fcgi->chroot) == -1) {
 				exit(EXIT_FAILURE);
@@ -61,12 +84,37 @@ int run_php_fcgi(t_php_fcgi *php_fcgi) {
 			exit(EXIT_FAILURE);
 		} while (false);
 
+		if (close(STDIN_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDIN\n");
+		} else if (open("/dev/null", O_RDONLY) == -1) {
+			fprintf(stderr, "Error redirecting stdin\n");
+			return EXIT_FAILURE;
+		}
+#ifndef DEBUG
+		if (close(STDOUT_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDOUT\n");
+		} else if (open("/dev/null", O_WRONLY) == -1) {
+			fprintf(stderr, "Error redirecting stdout\n");
+			return EXIT_FAILURE;
+		}
+		if (close(STDERR_FILENO) == -1) {
+			fprintf(stderr, "Warning: error closing STDERR\n");
+		} else if (open("/dev/null", O_WRONLY) == -1) {
+			log_string("Error redirecting stderr");
+			return EXIT_FAILURE;
+		}
+#endif
+
 		if (php_fcgi->config_file == NULL) {
 			execlp(php_fcgi->executable, php_fcgi->executable, "-b", php_fcgi->binding, (char*)NULL);
 		} else {
 			execlp(php_fcgi->executable, php_fcgi->executable, "-b", php_fcgi->binding, "-c", php_fcgi->config_file, (char*)NULL);
 		}
-		fprintf(stderr, "Can't execute %s\n", php_fcgi->executable);
+#ifdef DEBUG
+		printf(stderr, "Can't execute %s\n", php_fcgi->executable);
+#else
+		log_string("Can't execute php-cgi");
+#endif
 		exit(EXIT_FAILURE);
 	}
 
@@ -269,7 +317,7 @@ int main(int argc, char *argv[]) {
 
 			if ((fp = fopen(config->pidfile, "a")) != NULL) {
 				while (config->php_fcgi != NULL) {
-					if ((pid = run_php_fcgi(config->php_fcgi)) != -1) {
+					if ((pid = run_php_fcgi(config->php_fcgi, fp)) != -1) {
 						fprintf(fp, "%d\n", (int)pid);
 					} else {
 						fprintf(stderr, "Can't start FastCGI server\n");
