@@ -8,6 +8,7 @@
 #include <string.h>
 #include "rewrite.h"
 #include "libstr.h"
+#include "libfs.h"
 #include "alternative.h"
 
 #define MAX_RULE_LOOP   20
@@ -105,6 +106,13 @@ bool url_rewrite_setting(char *key, char *value, t_url_rewrite *rewrite) {
 						return false;
 					}
 				}
+			} else if (strcasecmp(value, "redirect") == 0) {
+				new_rule->action = rw_match_redirect;
+				if (strlen(rest) == 0) {
+					return false;
+				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
+					return false;
+				}
 			} else if (strcasecmp(value, "goto") == 0) {
 				new_rule->action = rw_match_goto;
 				if (strlen(rest) == 0) {
@@ -125,6 +133,11 @@ bool url_rewrite_setting(char *key, char *value, t_url_rewrite *rewrite) {
 				new_rule->action = rw_match_exit;
 			} else if (strcasecmp(value, "denyaccess") == 0) {
 				new_rule->action = rw_match_deny_access;
+			} else if (strcasecmp(value, "skip") == 0) {
+				new_rule->action = rw_match_skip;
+				if ((new_rule->value = str2int(rest)) < 1) {
+					return false;
+				}
 			} else {
 				return false;
 			}
@@ -135,6 +148,32 @@ bool url_rewrite_setting(char *key, char *value, t_url_rewrite *rewrite) {
 			} else if ((new_rule->parameter = strdup(value)) == NULL) {
 				return false;
 			}
+		} else if (strcmp(key, "skip") == 0) {
+			new_rule->action = rw_skip;
+			if ((new_rule->value = str2int(value)) < 1) {
+				return false;
+			}
+		} else if (strcmp(key, "requesturi") == 0) {
+			new_rule->action = rw_requesturi;
+			if (split_string(value, &value, &rest, ' ') == -1) {
+				return false;
+			}
+			if (strcasecmp(value, "exists") == 0) {
+				new_rule->value = IU_EXISTS;
+			} else if (strcasecmp(value, "isfile") == 0) {
+				new_rule->value = IU_ISFILE;
+			} else if (strcasecmp(value, "isdir") == 0) {
+				new_rule->value = IU_ISDIR;
+			} else {
+				return false;
+			}
+			if (strcasecmp(rest, "return") == 0) {
+				new_rule->after_rewrite = am_return;
+			} else if (strcasecmp(rest, "exit") == 0) {
+				new_rule->after_rewrite = am_exit;
+			} else {
+				return false;
+			}
 		} else {
 			return false;
 		}
@@ -247,11 +286,13 @@ static int do_rewrite(char *url, regex_t *regexp, char *rep, char **new_url, int
 	return 0;
 }
 
-int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **new_url) {
+int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **new_url, char *website_root) {
 	t_url_rewrite *rewrite;
 	t_rewrite_rule *rule;
 	bool replaced = false;
-	int loop = 0, result;
+	int loop = 0, result, skip = 0;
+	t_fsbool is_dir;
+	char *file, *qmark;
 
 	*new_url = NULL;
 	if ((rewrite = find_url_rewrite(rewrite_id, url_rewrite)) == NULL) {
@@ -260,7 +301,9 @@ int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **
 
 	rule = rewrite->rewrite_rule;
 	while (rule != NULL) {
-		switch (rule->action) {
+		if (skip > 0) {
+			skip--;
+		} else switch (rule->action) {
 			case rw_match_rewrite:
 				if (do_rewrite(url, &(rule->pattern), rule->parameter, new_url, rule->match_loop) == RW_ERROR) {
 					if (*new_url != NULL) {
@@ -288,6 +331,24 @@ int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **
 					*new_url = url;
 				}
 				break;
+			case rw_match_redirect:
+				if (do_rewrite(url, &(rule->pattern), rule->parameter, new_url, rule->match_loop) == RW_ERROR) {
+					if (*new_url != NULL) {
+						free(*new_url);
+						*new_url = NULL;
+					}
+					return RW_ERROR;
+				}
+				if (*new_url != NULL) {
+					if (replaced) {
+						free(url);
+					}
+					url = *new_url;
+					replaced = true;
+				} else if (replaced) {
+					*new_url = url;
+				}
+				return RW_REDIRECT;
 			case rw_match_goto:
 			case rw_match_call:
 			case rw_call:
@@ -300,7 +361,7 @@ int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **
 					return RW_ERROR;
 				}
 
-				if ((result = rewrite_url(url, rule->parameter, url_rewrite, new_url)) == RW_ERROR) {
+				if ((result = rewrite_url(url, rule->parameter, url_rewrite, new_url, website_root)) == RW_ERROR) {
 					if (*new_url != NULL) {
 						free(*new_url);
 						*new_url = NULL;
@@ -337,6 +398,46 @@ int rewrite_url(char *url, char *rewrite_id, t_url_rewrite *url_rewrite, char **
 					return RW_DENY_ACCESS;
 				}
 				break;
+			case rw_match_skip:
+				if (regexec(&(rule->pattern), url, 0, NULL, 0) == REG_NOMATCH) {
+					break;
+				}
+			case rw_skip:
+				skip = rule->value;
+				break;
+			case rw_requesturi:
+				if (valid_uri(url) == false) {
+					break;
+				}
+				if ((file = make_path(website_root, url)) == NULL) {
+					return RW_ERROR;
+				}
+
+				if ((qmark = strchr(file, '?')) != NULL) {
+					*qmark = '\0';
+				}
+				url_decode(file);
+				is_dir = is_directory(file);
+				free(file);
+
+				switch (rule->value) {
+					case IU_EXISTS:
+						if ((is_dir == yes) || (is_dir == no)) {
+							return (rule->after_rewrite == am_return ? RW_RETURN : RW_EXIT);
+						}
+						break;
+					case IU_ISFILE:
+						if (is_dir == no) {
+							return (rule->after_rewrite == am_return ? RW_RETURN : RW_EXIT);
+						}
+						break;
+					case IU_ISDIR:
+						if (is_dir == yes) {
+							return (rule->after_rewrite == am_return ? RW_RETURN : RW_EXIT);
+						}
+						break;
+				}
+				break;
 		}
 		rule = rule->next;
 	}
