@@ -34,8 +34,9 @@ char *hs_conn   = "Connection: ";               /* 12 */
 char *hs_concl  = "close\r\n";                  /*  7 */
 char *hs_conka  = "keep-alive\r\n";             /* 12 */
 char *hs_contyp = "Content-Type: ";             /* 14 */
-char *hs_lctn   = "Location: http";             /* 14 */
-char *hs_slash  = "://";                        /*  3 */
+char *hs_lctn   = "Location: ";                 /* 10 */
+char *hs_http   = "http://";                    /*  7 */
+char *hs_https  = "https://";                   /*  8 */
 char *hs_range  = "Accept-Ranges: bytes\r\n";   /* 22 */
 char *hs_gzip   = "Content-Encoding: gzip\r\n"; /* 24 */
 char *hs_eol    = "\r\n";                       /*  2 */
@@ -204,12 +205,14 @@ int send_header(t_session *session, int code) {
 		return -1;
 	} else if (send_buffer(session, timestr, strlen(timestr)) == -1) {
 		return -1;
-	} else if (send_buffer(session, hs_server, 8) == -1) {
-		return -1;
-	} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
-		return -1;
-	} else if (send_buffer(session, hs_eol, 2) == -1) {
-		return -1;
+	} else if (strcmp(session->config->server_string, "-") != 0) {
+		if (send_buffer(session, hs_server, 8) == -1) {
+			return -1;
+		} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
+			return -1;
+		} else if (send_buffer(session, hs_eol, 2) == -1) {
+			return -1;
+		}
 	} else if ((session->cgi_type == no_cgi) && (session->uri_is_dir == false)) {
 		if (send_buffer(session, hs_range, 22) == -1) {
 			return -1;
@@ -328,17 +331,30 @@ int send_code(t_session *session, int code) {
 
 	switch (code) {
 		case 301:
-			if (send_buffer(session, hs_lctn, 14) == -1) {
+			if (send_buffer(session, hs_lctn, 10) == -1) {
 				return -1;
 			}
-#ifdef HAVE_SSL
-			if (session->binding->use_ssl || (session->cause_of_301 == require_ssl)) {
-				if (send_buffer(session, "s", 1) == -1) {
+
+			if (session->cause_of_301 == location) {
+				if (session->location != NULL) {
+					if (send_buffer(session, session->location, strlen(session->location)) == -1) {
+						return -1;
+					}
+				}
+				if (send_buffer(session, "\r\n", 2) == -1) {
 					return -1;
 				}
+				break;
 			}
+
+#ifdef HAVE_SSL
+			if (session->binding->use_ssl || (session->cause_of_301 == require_ssl)) {
+				if (send_buffer(session, hs_https, 8) == -1) {
+					return -1;
+				}
+			} else
 #endif
-			if (send_buffer(session, hs_slash, 3) == -1) {
+			if (send_buffer(session, hs_http, 7) == -1) {
 				return -1;
 			}
 
