@@ -17,7 +17,7 @@
 #endif
 
 typedef enum { root, part } t_pathmatch;
-typedef enum { basic, digest } t_auth_method;
+typedef enum { no_auth, basic, digest } t_auth_method;
 typedef enum { hiawatha, common, extended } t_log_format;
 #ifdef CYGWIN
 typedef enum { windows, cygwin } t_platform;
@@ -145,6 +145,7 @@ typedef struct type_host {
 	t_charlist    hostname;
 	bool          user_websites;
 	bool          execute_cgi;
+	int           time_for_cgi;
 	bool          show_index;
 	bool          use_gz_file;
 	char          *login_message;
@@ -162,6 +163,7 @@ typedef struct type_host {
 	t_accesslist  *access_list;
 	t_accesslist  *alter_list;
 	mode_t        alter_fmode;
+	char          *run_on_alter;
 	t_charlist    image_referer;
 	char          *imgref_replacement;
 	t_keyvalue    *envir_str;
@@ -205,7 +207,6 @@ typedef struct type_config {
 	t_charlist    cgi_extension;
 	int           total_connections;
 	int           connections_per_ip;
-	int           time_for_cgi;
 	bool          kill_timedout_cgi;
 	char          *system_logfile;
 	char          *garbage_logfile;
