@@ -70,6 +70,7 @@ static void clear_session(t_session *session) {
 	session->tempdata = NULL;
 	session->uploaded_file = NULL;
 	session->uploaded_size = 0;
+	session->location = NULL;
 #ifdef DEBUG
 	session->status = NULL;
 #endif
@@ -108,6 +109,7 @@ void reset_session(t_session *session) {
 	sfree(session->remote_user);
 	sfree(session->path_info);
 	sfree(session->request_uri);
+	sfree(session->location);
 	if (session->uploaded_file != NULL) {
 		unlink(session->uploaded_file);
 		free(session->uploaded_file);
@@ -115,7 +117,7 @@ void reset_session(t_session *session) {
 #ifdef DEBUG
 	sfree(session->status);
 #endif
-	remove_headerfields(session);
+	session->headerfields = remove_headerfields(session->headerfields);
 	if (session->directory != NULL) {
 		pthread_mutex_lock(&(session->directory->client_mutex));
 		if (session->part_of_dirspeed) {
@@ -192,87 +194,6 @@ void determine_request_method(t_session *session) {
 	}
 }
 
-/* Parse the HTTP headerfields from the received request.
- */
-t_headerfield *parse_headerfields(char *line) {
-	t_headerfield *first, *headerfield;
-	char *value;
-
-	if (*line == '\0') {
-		return NULL;
-	}
-
-	if ((first = headerfield = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
-		return NULL;
-	}
-	headerfield->data = line;
-	headerfield->next = NULL;
-	while (*line != '\0') {
-		if (*line == '\r') {
-			*line = '\0';
-			if (*(line + 1) == '\n') {
-				if ((*(line + 2) != '\r') && (*(line + 2) != '\0')) {
-					if ((headerfield->next = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
-						return first;
-					}
-					headerfield = headerfield->next;
-					headerfield->next = NULL;
-					headerfield->data = line + 2;
-				} else {
-					break;
-				}
-			}
-		}
-		line++;
-	}
-
-	headerfield = first;
-	while (headerfield != NULL) {
-		if ((value = strchr(headerfield->data, ':')) != NULL) {
-			do {
-				value++;
-			} while ((*value == ' ') && (*value != '\0'));
-			headerfield->value_offset = (value - headerfield->data);
-		} else {
-			headerfield->value_offset = 0;
-		}
-		headerfield = headerfield->next;
-	}
-
-	return first;
-}
-
-/* Search for a headerfield and return its value.
- */
-char *get_headerfield(char *key, t_headerfield *headerfields) {
-	char *retval = NULL;
-	int len;
-
-	len = strlen(key);
-	while (headerfields != NULL) {
-		if (strncasecmp(headerfields->data, key, len) == 0) {
-			retval = headerfields->data + headerfields->value_offset;
-			break;
-		}
-		headerfields = headerfields->next;
-	}
-
-	return retval;
-}
-
-/* free() a list of headerfields.
- */
-void remove_headerfields(t_session *session) {
-	t_headerfield *to_be_removed;
-
-	while (session->headerfields != NULL) {
-		to_be_removed = session->headerfields;
-		session->headerfields = session->headerfields->next;
-
-		free(to_be_removed);
-	}
-}
-
 /* Return the path of the user's homedirectory.
  */
 int get_homedir(t_session *session, char *username) {
@@ -480,7 +401,7 @@ void close_socket(t_session *session) {
 	if (session->socket_open) {
 #ifdef HAVE_SSL
 		if (session->binding->use_ssl) {
-			ssl_close(session->ssl_data);
+			ssl_close(&(session->ssl_data));
 		}
 #endif
 		fsync(session->client_socket);
