@@ -15,16 +15,9 @@
 typedef enum { no_cgi, program, script, fastcgi} t_cgi_type;
 typedef enum { unknown, GET, POST, HEAD, OPTIONS, TRACE, PUT, DELETE, unsupported } t_req_method;
 #ifdef HAVE_SSL
-typedef enum { missing_slash, require_ssl } t_cause_of_301;
+typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
 #endif
 
-typedef struct type_headerfield {
-	char *data;
-	int  value_offset;
-
-	struct type_headerfield *next;
-} t_headerfield;
-
 typedef struct type_session {
 	t_config        *config;
 
@@ -67,6 +60,7 @@ typedef struct type_session {
 	t_tempdata      *tempdata;
 	char            *uploaded_file;
 	long            uploaded_size;
+	char            *location;
 
 	/* Throttling: send_buffer() in send.c
 	 */
@@ -101,9 +95,6 @@ void reset_session(t_session *session);
 void destroy_session(t_session *session);
 
 void determine_request_method(t_session *session);
-t_headerfield *parse_headerfields(char *line);
-char *get_headerfield(char *key, t_headerfield *headerfields);
-void remove_headerfields(t_session *session);
 
 int  get_homedir(t_session *session, char *username);
 bool duplicate_host(t_session *session);
