@@ -58,6 +58,7 @@ char *hs_chunked = "Transfer-Encoding: chunked\r\n";
 char *hs_allow   = "Allow: GET, HEAD, OPTIONS";
 char *hs_post    = ", POST";
 char *hs_trace   = ", TRACE";
+char *hs_alter   = ", PUT, DELETE";
 
 extern char *fb_filesystem;
 extern char *fb_symlink;
@@ -1199,6 +1200,11 @@ int handle_options_request(t_session *session) {
 			return -1;
 		}
 	}
+	if (session->binding->enable_alter) {
+		if (send_buffer(session, hs_alter, 13) == -1) {
+			return -1;
+		}
+	}
 	if (send_buffer(session, hs_eol, 2) == -1) {
 		return -1;
 	} else if (send_buffer(session, hs_conlen, 16) == -1) {
