@@ -12,7 +12,7 @@
 #include "rewrite.h"
 #endif
 
-#define MAX_INPUT KILOBYTE
+#define MAX_INPUT_SIZE KILOBYTE
 
 typedef struct type_line {
 	char *key, *value, *file;
@@ -147,7 +147,7 @@ int read_directory(char *dir, t_line **config, t_line **aliases) {
 
 int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_include) {
 	FILE *fp;
-	char line[MAX_INPUT + 1], *data, *value;
+	char line[MAX_INPUT_SIZE + 1], *data, *value;
 	bool is_alias;
 	int linenr = 0, retval = 0;
 
@@ -160,8 +160,8 @@ int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_
 		return -1;
 	}
 
-	line[MAX_INPUT] = '\0';
-	while (fgets(line, MAX_INPUT, fp) != NULL) {
+	line[MAX_INPUT_SIZE] = '\0';
+	while (fgets(line, MAX_INPUT_SIZE, fp) != NULL) {
 		linenr++;
 		data = strip_line(line);
 		if ((data[0] == '#') || (data[0] == '\0')) {
@@ -457,10 +457,10 @@ int check_main_config(char *config_dir) {
 #ifdef HAVE_REWRITE
 void check_url_rewrite(char *config_dir, char **rewrite_id) {
 	t_line *config = NULL;
-	char input[MAX_INPUT + 1], **id, *url, *new_url;
+	char input[MAX_INPUT_SIZE + 1], **id, *url, *new_url;
 	t_url_rewrite *url_rewrite, *rewrite = NULL;
 	bool in_rule_section = false;
-	int result;
+	int result = 0;
 
 	if (chdir(config_dir) == -1) {
 		perror(config_dir);
@@ -526,10 +526,10 @@ void check_url_rewrite(char *config_dir, char **rewrite_id) {
 
 	/* Start testing
 	 */
-	input[MAX_INPUT] = '\0';
+	input[MAX_INPUT_SIZE] = '\0';
 	printf("\n===[ URL rewrite tester\n");
 	printf("Use empty input to leave the program.\n\nurl: ");
-	while (fgets(input, MAX_INPUT, stdin) != NULL) {
+	while (fgets(input, MAX_INPUT_SIZE, stdin) != NULL) {
 		url = remove_spaces(input);
 
 		if (strcmp(url, "") == 0) {
@@ -543,12 +543,15 @@ void check_url_rewrite(char *config_dir, char **rewrite_id) {
 
 		id = rewrite_id;
 		while (*id != NULL) {
-			if ((result = rewrite_url(url, *id, url_rewrite, &new_url)) == RW_ERROR) {
+			if ((result = rewrite_url(url, *id, url_rewrite, &new_url, ".")) == RW_ERROR) {
 				perror("rewrite_url()");
 			}
 			if (new_url != NULL) {
 				url = new_url;
 			}
+			if (result == RW_REDIRECT) {
+				break;
+			}
 			if (result == RW_DENY_ACCESS) {
 				url = "(403 Forbidden)";
 				break;
@@ -557,7 +560,9 @@ void check_url_rewrite(char *config_dir, char **rewrite_id) {
 		}
 
 		if (new_url != NULL) {
-			if (*new_url != '/') {
+			if (result == RW_REDIRECT) {
+				printf("Request is redirected.\n");
+			} else if (*new_url != '/') {
 				printf("Warning: your new URL is missing a leading slash!\n");
 			}
 			printf("new: %s\n\n", new_url);
